@@ -1,14 +1,15 @@
+from bespin.amazon.s3 import delete_key_from_s3, list_keys_from_s3_path, upload_file_to_s3
 from bespin.errors import NoSuchStack
 from bespin.layers import Layers
 from bespin import helpers as hp
-from bespin import aws_helpers as aws
 
 import logging
+import os
 
 log = logging.getLogger("bespin.actions.builder")
 
 class Builder(object):
-    def deploy_stack(self, stack, stacks, made=None, ignore_deps=False):
+    def deploy_stack(self, stack, stacks, credentials, made=None, ignore_deps=False):
         """Make us an stack"""
         made = made or {}
 
@@ -46,32 +47,50 @@ class Builder(object):
         print("Building - {0}".format(stack.stack_name))
         print(stack.params_json_obj)
 
-    def publish_artifacts(self, stack):
-        # Iterate over each artifact we need to build
-        for key, artifact in stack.artifacts.items():
-            # Check we are not missing any env vars
+    def find_missing_build_env(self, stack):
+        for artifact in stack.artifacts.values():
             artifact.find_missing_env()
 
+    def publish_artifacts(self, stack, credentials):
+        # Find missing env before doing anything
+        self.find_missing_build_env(stack)
+
+        # Iterate over each artifact we need to build
+        for key, artifact in stack.artifacts.items():
             # Gather our environment variables
             environment = dict(env.pair for env in artifact.build_env)
 
             # Create a temporary file to tar to
             with hp.a_temp_file() as temp_tar_file:
                 # Make the artifact
-                hp.generate_tar_file(temp_tar_file, artifact.paths, artifact.files
+                hp.generate_tar_file(temp_tar_file, artifact.paths + artifact.files
                     , environment=environment
                     , compression=artifact.compression_type
                     )
                 log.info("Finished generating artifact: {0}".format(key))
 
                 # Upload the artifact
-                aws.upload_file_to_s3(temp_tar_file.name, artifact.upload_to.format(**environment))
+                upload_file_to_s3(credentials, temp_tar_file.name, artifact.upload_to.format(**environment))
+
+    def clean_old_artifacts(self, stack, credentials):
+        # Find missing env before doing anything
+        self.find_missing_build_env(stack)
 
-    def clean_old_artifacts(self, stack):
         # Iterate over each artifact we need to clean
         for key, artifact in stack.artifacts.items():
-            # Check we are not missing any env vars
-            artifact.find_missing_env()
+            environment = dict(env.pair for env in artifact.build_env)
 
-            # Clean it
-            print(artifact.history_length)
+            # Get contents of bucket
+            artifact_path = os.path.dirname(artifact.upload_to.format(**environment))
+            artifact_keys = list_keys_from_s3_path(credentials, artifact_path)
+
+            # Get all the time stamps and determine the files to delete
+            timestamps = list(map(lambda x: x.last_modified, artifact_keys))
+            timestamps.sort()
+            keys_to_del = timestamps[:-artifact.history_length]
+
+            # Iterate through all the artifacts deleting any ones flagged for deletion
+            for artifact_key in artifact_keys:
+                if artifact_key.last_modified in keys_to_del:
+                    log.info("Deleting artifact %s ", artifact_key.name)
+                    delete_key_from_s3(credentials, artifact_key, stack.bespin.dry_run)
